@@ -1,45 +1,24 @@
 #TODO:
-from concurrent.futures import ThreadPoolExecutor, as_completed
+#from concurrent.futures import ThreadPoolExecutor, as_completed
 from copy import deepcopy
 from datetime import datetime
 from glob import glob
 import gzip
 from fastnumbers import isint, isfloat, fast_int, fast_float
-#TODO:
 from itertools import chain
 from joblib import Parallel, delayed
 from math import ceil, log
-from mmap import mmap, PROT_READ
+#TODO: Remove mmap.ACCESS_READ?
+from mmap import mmap, PROT_READ, ACCESS_READ
 from msgspec import msgpack
 from operator import eq, ge, gt, le, lt, ne, itemgetter
-from os import makedirs, path
+from os import makedirs, path, remove
 from re import compile
 from shutil import rmtree
 import sys
 from tempfile import mkdtemp
 from zstandard import ZstdCompressor, ZstdDecompressor
 
-# import concurrent.futures
-# import copy
-# import datetime
-# import glob
-# import gzip
-# import fastnumbers
-# #TODO:
-# from itertools import chain
-# from joblib import Parallel, delayed
-# import math
-# import mmap
-# import msgspec
-# import operator
-# from operator import itemgetter
-# import os
-# import re
-# import shutil
-# import sys
-# import tempfile
-# import zstandard
-
 # We use these dictionaries so that when we store the file map, it takes less space on disk.
 FILE_KEY_ABBREVIATIONS_STATS = {"mccl": 3, "mctl": 6, "cnmccl": 10, "cnll": 11}
 FILE_KEY_ABBREVIATIONS_OTHER = {"ll": 4, "cmpr": 7}
@@ -47,7 +26,8 @@ FILE_KEY_ABBREVIATIONS_NOCACHE = {"data": 1, "cc": 2, "ct": 5, "cndata": 8, "cnc
 
 def open_read_file(file_path, file_extension=""):
     the_file = open(file_path + file_extension, 'rb')
-    return mmap(the_file.fileno(), 0, prot=PROT_READ)
+    #return mmap(the_file.fileno(), 0, prot=PROT_READ)
+    return mmap(the_file.fileno(), 0, access=ACCESS_READ)
 
 def read_str_from_file(file_path, file_extension=""):
     with open(file_path + file_extension, 'rb') as the_file:
@@ -158,25 +138,6 @@ def deserialize(msg):
     return msgpack.decode(msg)
 #    return msgpack.unpackb(msg, strict_map_key=False)
 
-def prepare_tmp_dirs(tmp_dir_path):
-    # Figure out where temp files will be stored and create directory, if needed.
-    if tmp_dir_path:
-        makedirs(tmp_dir_path, exist_ok=True)
-    else:
-        tmp_dir_path = mkdtemp()
-
-    tmp_dir_path = fix_dir_path_ending(tmp_dir_path)
-
-    tmp_dir_path_chunks = f"{tmp_dir_path}chunks/"
-    tmp_dir_path_outputs = f"{tmp_dir_path}outputs/"
-    tmp_dir_path_indexes = f"{tmp_dir_path}indexes/"
-
-    makedirs(tmp_dir_path_chunks, exist_ok=True)
-    makedirs(tmp_dir_path_outputs, exist_ok=True)
-    makedirs(tmp_dir_path_indexes, exist_ok=True)
-
-    return tmp_dir_path_chunks, tmp_dir_path_outputs, tmp_dir_path_indexes
-
 def fix_dir_path_ending(dir_path):
     return dir_path if dir_path.endswith("/") else dir_path + "/"
 
